@@ -1,6 +1,6 @@
 /* c_size.c
  *
- * Copyright (c) 2002 Mike Gleason, NcFTP Software.
+ * Copyright (c) 1996-2005 Mike Gleason, NcFTP Software.
  * All rights reserved.
  *
  */
@@ -57,7 +57,7 @@ FTPFileSize(const FTPCIPtr cip, const char *const file, longest_int *const size,
 #endif
 				cip->hasSIZE = kCommandAvailable;
 				result = kNoErr;
-			} else if (UNIMPLEMENTED_CMD(rp->code)) {
+			} else if (FTP_UNIMPLEMENTED_CMD(rp->code)) {
 				cip->hasSIZE = kCommandNotAvailable;
 				cip->errNo = kErrSIZENotAvailable;
 				result = kErrSIZENotAvailable;
@@ -70,3 +70,69 @@ FTPFileSize(const FTPCIPtr cip, const char *const file, longest_int *const size,
 	}
 	return (result);
 }	/* FTPFileSize */
+
+
+
+
+longest_int
+FTPLocalASCIIFileSize(const char *const fn, char *buf, const size_t bufsize)
+{
+	char *tbuf = NULL;
+	int c, prevc;
+	longest_int asize;
+	const char *scp, *slim;
+	int fd;
+	read_return_t nread;
+	int oerrno;
+
+	if (buf == NULL) {
+		tbuf = (char *) malloc(bufsize);
+		if (tbuf == NULL)
+			return ((longest_int) -1);
+		buf = tbuf;
+	}
+
+	fd = Open(fn, O_RDONLY, 00666);
+	if (fd < 0) {
+		if (tbuf != NULL)
+			free(tbuf);
+		return ((longest_int) -1);
+	}
+
+	prevc = 0;
+	for (asize = (longest_int) 0; ; asize += (longest_int) nread) {
+		nread = read(fd, buf, bufsize);
+		if (nread < 0) {
+			oerrno = errno;
+			(void) close(fd);
+			if (tbuf != NULL)
+				free(tbuf);
+			errno = oerrno;
+			return ((longest_int) -1);
+		} else if (nread == 0) {
+			break;
+		}
+
+		for (scp = buf, slim = buf + nread; scp < slim; ) {
+			c = *scp++;
+			if ((c == '\n') && (prevc != '\r')) {
+				/* When we actually use
+				 * the file later, we translate
+				 * newlines into newline+carriage return,
+				 * so add one to the count for the
+				 * CR byte we'll add then.
+				 */
+				nread++;
+			}
+			prevc = c;
+		}
+	}
+
+	if (tbuf != NULL)
+		free(tbuf);
+	(void) close(fd);
+	return (asize);
+}	/* FTPLocalASCIIFileSize */
+
+
+
