@@ -113,6 +113,9 @@ static const char *gErrList[kErrLast - kErrFirst + 2] = {
 	"data connection did not originate from correct port number",	/* -199 */
 	"internal error: bug in library code detected",			/* -200 */
 	"remote server does not support setting a RESTart point",	/* -201 */
+	"miscellaneous error occurred while trying to open the host", 	/* -202 */
+	"miscellaneous error occurred while trying to login to the host", /* -203 */
+	"ascii seek error",						/* -204 */
 	NULL,								
 };
 
