@@ -568,7 +568,7 @@ CompletionFunction(const char *text, int state)
 		bUsed = MakeArgv(cmdstart, &ai.cargc, ai.cargv,
 			(int) (sizeof(ai.cargv) / sizeof(char *)),
 			ai.argbuf, sizeof(ai.argbuf),
-			ai.noglobargv, 1);
+			ai.noglobargv, 1, 1);
 		if (bUsed <= 0)
 			return NULL;
 		if (ai.cargc == 0)
@@ -600,6 +600,7 @@ CompletionFunction(const char *text, int state)
 		cp = RemoteDirCompletionFunction(text, state);
 		return cp;
 	} else if ((flags & kCompleteBookmark) != 0) {
+		gl_filename_quoting_desired = 1;
 		cp = BookmarkCompletionFunction(text, state);
 		return cp;
 	} else if ((flags & kCompletePrefOpt) != 0) {
