@@ -953,7 +953,7 @@ LogEndItemResult(int uiShow, const char *const fmt, ...)
 	va_list ap;
 	char buf[512];
 
-	strcpy(buf, "\nresult=");
+	(void) strcpy(buf, "\nresult=");
 
 	va_start(ap, fmt);
 #ifdef HAVE_VSNPRINTF
@@ -1068,6 +1068,8 @@ DoItem(void)
 		gConn.port = gPort;
 		(void) STRNCPY(gConn.user, gRUser);
 		(void) STRNCPY(gConn.pass, gRPass);
+		if ((gConn.pass[0] == '\0') && (strcmp(gConn.user, "anonymous")) && (strcmp(gConn.user, "ftp")) && (gConn.user[0] != '\0'))
+			gConn.passIsEmpty = 1;
 		(void) STRNCPY(gConn.acct, gRAcct);
 		gConn.maxDials = 1;
 		gConn.dataPortMode = gPassive;
@@ -1157,7 +1159,7 @@ DoItem(void)
 		if (result == kErrCouldNotStartDataTransfer) {
 			LogEndItemResult(1, "Remote item %s is no longer retrievable.\n", gRFile);
 			result = 0;	/* file no longer on host */
-		} else if (result == kErrRemoteSameAsLocal) {
+		} else if ((result == kErrRemoteSameAsLocal) || (result == kErrLocalSameAsRemote)) {
 			LogEndItemResult(1, "Remote item %s is already present locally.\n", gRFile);
 			result = 0;
 		} else if (result == kErrLocalFileNewer) {
@@ -1184,7 +1186,7 @@ DoItem(void)
 		if (result == kErrCouldNotStartDataTransfer) {
 			LogEndItemResult(1, "Remote item %s is no longer sendable.  Perhaps permission denied on destination?\n", gRFile);
 			result = 0;	/* file no longer on host */
-		} else if (result == kErrLocalSameAsRemote) {
+		} else if ((result == kErrRemoteSameAsLocal) || (result == kErrLocalSameAsRemote)) {
 			LogEndItemResult(1, "Local item %s is already present on remote host.\n", gLFile);
 			result = 0;
 		} else if (result == kErrRemoteFileNewer) {
