@@ -63,6 +63,7 @@ Usage(void)
   -t XX  Timeout after XX seconds.\n\
   -v/-V  Do (do not) use progress meters.\n\
   -f XX  Read the file XX for host, user, and password information.\n\
+  -h XX  Connect to host XX.  Useful for overriding host in -f config.file.\n\
   -c     Read from remote host and write locally to stdout.\n\
   -C     Read from remote host and write locally to specified file.\n\
   -A     Append to local files, instead of overwriting them.\n");
@@ -227,7 +228,7 @@ main(int argc, char **argv)
 	perfilecmd[0] = '\0';
 
 	GetoptReset(&opt);
-	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:e:d:t:aRTr:vVf:ADzZEFbcCB:W:X:Y:")) > 0) {
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:t:aRTr:vVf:ADzZEFbcCB:W:X:Y:")) > 0) {
 		if (c == 'b') {
 			batchmode++;
 		}
@@ -235,7 +236,7 @@ main(int argc, char **argv)
 
 	if (batchmode > 0) {
 		GetoptReset(&opt);
-		while ((c = Getopt(&opt, argc, argv, "P:u:j:p:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) switch(c) {
+		while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) switch(c) {
 			case 'v': case 'V': case 'A': case 'B': case 'T':
 			case 'd': case 'e': case 't': case 'r': case 'c':
 			case 'z': case 'Z': case 'C':
@@ -246,7 +247,7 @@ main(int argc, char **argv)
 	}
 
 	GetoptReset(&opt);
-	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:e:d:t:aRTr:vVf:o:ADzZEFbcCB:W:X:Y:")) > 0) switch(c) {
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:t:aRTr:vVf:o:ADzZEFbcCB:W:X:Y:")) > 0) switch(c) {
 		case 'P':
 			fi.port = atoi(opt.arg);	
 			break;
@@ -262,9 +263,16 @@ main(int argc, char **argv)
 			break;
 		case 'p':
 			(void) STRNCPY(fi.pass, opt.arg);	/* Don't recommend doing this! */
+			if (fi.pass[0] == '\0')
+				fi.passIsEmpty = 1;
 			memset(opt.arg, 0, strlen(fi.pass));
 			opt.arg[0] = '?';
 			break;
+		case 'h':
+			(void) STRNCPY(fi.host, opt.arg);
+			memset(opt.arg, 0, strlen(fi.user));
+			opt.arg[0] = '?';
+			break;
 		case 'e':
 			if (strcmp(opt.arg, "stdout") == 0)
 				fi.errLog = stdout;
@@ -364,6 +372,9 @@ main(int argc, char **argv)
 	if (progmeters != 0)
 		fi.progress = PrStatBar;
 
+	if ((perfilecmd[0] != '\0') && (rflag != 0))
+		(void) fprintf(stderr, "Warning: your -X command is only applied once per command-line parameter, and not for each file in the directory.\n");
+
 	if (fi.host[0] == '\0') {
 		(void) STRNCPY(url, argv[opt.ind]);
 		rc = FTPDecodeURL(&fi, url, &cdlist, urlfile, sizeof(urlfile), (int *) &urlxtype, NULL);
@@ -453,7 +464,7 @@ main(int argc, char **argv)
 	}
 
 	if (strcmp(fi.user, "anonymous") && strcmp(fi.user, "ftp")) {
-		if (fi.pass[0] == '\0') {
+		if ((fi.pass[0] == '\0') && (fi.passIsEmpty == 0)) {
 			(void) gl_getpass("Password: ", fi.pass, sizeof(fi.pass));
 		}
 	}
