@@ -65,6 +65,7 @@ Usage(void)
   -v/-V  Do (do not) use progress meters.\n\
   -f XX  Read the file XX for host, user, and password information.\n");
 	(void) fprintf(fp, "\
+  -h XX  Connect to host XX.  Useful for overriding host in -f config.file.\n\
   -c     Read locally from stdin and write remotely to specified pathname.\n\
   -C     Similar to -c, except a local pathname is specified.\n\
   -A     Append to remote files instead of overwriting them.\n\
@@ -185,6 +186,7 @@ main(int argc, char **argv)
 	int batchmode = 0;
 	int spooled = 0;
 	int i;
+	int ascii = 0;
 	char *ufilep;
 	const char *udirp;
 	char ufile[256];
@@ -228,7 +230,7 @@ main(int argc, char **argv)
 	perfilecmd[0] = '\0';
 
 	GetoptReset(&opt);
-	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) {
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) {
 		if (c == 'b') {
 			batchmode++;
 		}
@@ -236,7 +238,7 @@ main(int argc, char **argv)
 
 	if (batchmode > 0) {
 		GetoptReset(&opt);
-		while ((c = Getopt(&opt, argc, argv, "P:u:j:p:e:d:U:t:mar:RvVf:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) switch(c) {
+		while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) switch(c) {
 			case 'v': case 'V': case 'A': case 'B': case 'S':
 			case 'T': case 'd': case 'e': case 'U': case 't':
 			case 'm': case 'r': case 'c': case 'y': case 'z':
@@ -248,7 +250,7 @@ main(int argc, char **argv)
 	}
 
 	GetoptReset(&opt);
-	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) switch(c) {
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) switch(c) {
 		case 'P':
 			fi.port = atoi(opt.arg);	
 			break;
@@ -264,9 +266,16 @@ main(int argc, char **argv)
 			break;
 		case 'p':
 			(void) STRNCPY(fi.pass, opt.arg);	/* Don't recommend doing this! */
+			if (fi.pass[0] == '\0')
+				fi.passIsEmpty = 1;
 			memset(opt.arg, 0, strlen(fi.pass));
 			opt.arg[0] = '?';
 			break;
+		case 'h':
+			(void) STRNCPY(fi.host, opt.arg);
+			memset(opt.arg, 0, strlen(fi.user));
+			opt.arg[0] = '?';
+			break;
 		case 'e':
 			if (strcmp(opt.arg, "stdout") == 0)
 				fi.errLog = stdout;
@@ -302,6 +311,7 @@ main(int argc, char **argv)
 			break;
 		case 'a':
 			xtype = kTypeAscii;	/* Use ascii. */
+			ascii++;
 			break;
 		case 'r':
 			SetRedial(&fi, opt.arg);
@@ -374,6 +384,9 @@ main(int argc, char **argv)
 			Usage();
 	}
 
+	if ((perfilecmd[0] != '\0') && (rflag != 0))
+		(void) fprintf(stderr, "Warning: your -X command is only applied once per command-line parameter, and not for each file in the directory.\n");
+
 	if (usingcfg != 0) {
 		if (ftpcat == 0) {
 			if (opt.ind > argc - 2)
@@ -388,7 +401,7 @@ main(int argc, char **argv)
 		} else {
 			if (opt.ind > argc - 1)
 				Usage();
-			dstfile = argv[opt.ind + 1];
+			dstfile = argv[opt.ind + 0];
 		}
 	} else {
 		if (ftpcat == 0) {
@@ -412,7 +425,7 @@ main(int argc, char **argv)
 	}
 
 	if (strcmp(fi.user, "anonymous") && strcmp(fi.user, "ftp")) {
-		if (fi.pass[0] == '\0') {
+		if ((fi.pass[0] == '\0') && (fi.passIsEmpty == 0)) {
 			(void) gl_getpass("Password: ", fi.pass, sizeof(fi.pass));
 		}
 	}
@@ -423,6 +436,12 @@ main(int argc, char **argv)
 		fi.hasSITE_UTIME = 0;
 	if (nD >= 2)
 		deleteflag = kDeleteYes;
+	if (ascii > 1) {
+		/* This is used internally for testing.
+		 * Valid values as of this release are -1, 0, 1.
+		 */
+		fi.asciiTranslationMode = ascii - 3;
+	}
 
 	if (MayUseFirewall(fi.host, gFirewallType, gFirewallExceptionList) != 0) {
 		fi.firewallType = gFirewallType; 
@@ -477,7 +496,7 @@ main(int argc, char **argv)
 				0
 			);
 			if ((result == 0) && (batchmode < 3)) {
-				fprintf(stdout, "  + Spooled; sending remotely as %s/%s.\n", dstdir, ufilep);
+				(void) fprintf(stdout, "  + Spooled; sending remotely as %s/%s.\n", dstdir, ufilep);
 				spooled++;
 			}
 		}
@@ -558,9 +577,14 @@ main(int argc, char **argv)
 			}
 		} else {
 			fi.progress = (FTPProgressMeterProc) 0;
-			if (FTPPutOneFile3(&fi, NULL, dstfile, xtype, STDIN_FILENO, appendflag, tmppfx, tmpsfx, resumeflag, deleteflag, kNoFTPConfirmResumeUploadProc, 0) < 0)
+			result = kNoErr;
+			if (wantMkdir)
+				result = FTPMkParentDir(&fi, dstfile, 1, NULL);
+			if (result != kNoErr) {
+				es = kExitMkdirFailed;
+			} else if (FTPPutOneFile3(&fi, NULL, dstfile, xtype, STDIN_FILENO, appendflag, tmppfx, tmpsfx, resumeflag, deleteflag, kNoFTPConfirmResumeUploadProc, 0) < 0) {
 				es = kExitXferFailed;
-			else {
+			} else {
 				es = kExitSuccess;
 				(void) AdditionalCmd(&fi, perfilecmd, argv[opt.ind + 1]);
 			}
