@@ -3,77 +3,6 @@
 #	pragma hdrstop
 #endif
 
-#ifndef NO_SIGNALS
-extern Sjmp_buf gNetTimeoutJmp;
-extern Sjmp_buf gPipeJmp;
-#endif
-
-#ifndef NO_SIGNALS
-
-int
-SSendtoByName(int sfd, const char *const buf, size_t size, int fl, const char *const toAddrStr, int tlen)
-{
-	send_return_t nwrote;
-	alarm_time_t tleft;
-	int result;
-	vsio_sigproc_t sigalrm, sigpipe;
-	time_t done, now;
-	struct sockaddr_in toAddr;
-
-	if ((result = AddrStrToAddr(toAddrStr, &toAddr, -1)) < 0) {
-		return (result);
-	}
-
-	if (SSetjmp(gNetTimeoutJmp) != 0) {
-		alarm(0);
-		(void) SSignal(SIGALRM, (sio_sigproc_t) sigalrm);
-		(void) SSignal(SIGPIPE, (sio_sigproc_t) sigpipe);
-		errno = ETIMEDOUT;
-		return (kTimeoutErr);
-	}
-
-	if (SSetjmp(gPipeJmp) != 0) {
-		alarm(0);
-		(void) SSignal(SIGALRM, (sio_sigproc_t) sigalrm);
-		(void) SSignal(SIGPIPE, (sio_sigproc_t) sigpipe);
-		errno = EPIPE;
-		return (kBrokenPipeErr);
-	}
-
-	sigalrm = (vsio_sigproc_t) SSignal(SIGALRM, SIOHandler);
-	sigpipe = (vsio_sigproc_t) SSignal(SIGPIPE, SIOHandler);
-
-	time(&now);
-	done = now + tlen;
-	tleft = (done > now) ? ((alarm_time_t) (done - now)) : 0;
-	forever {
-		(void) alarm(tleft);
-		nwrote = sendto(sfd, buf, (send_size_t) size, fl,
-				(struct sockaddr *) &toAddr,
-				(sockaddr_size_t) sizeof(struct sockaddr_in));
-		(void) alarm(0);
-		if (nwrote >= 0)
-			break;
-		if (errno != EINTR)
-			break;		/* Fatal error. */
-		errno = 0;
-		time(&now);
-		tleft = (done > now) ? ((alarm_time_t) (done - now)) : 0;
-		if (tleft < 1) {
-			nwrote = kTimeoutErr;
-			errno = ETIMEDOUT;
-			break;
-		}
-	}
-
-	(void) SSignal(SIGALRM, (sio_sigproc_t) sigalrm);
-	(void) SSignal(SIGPIPE, (sio_sigproc_t) sigpipe);
-
-	return ((int) nwrote);
-}	/* SSendtoByName */
-
-#else
-
 int
 SSendtoByName(int sfd, const char *const buf, size_t size, int fl, const char *const toAddrStr, int tlen)
 {
@@ -83,6 +12,12 @@ SSendtoByName(int sfd, const char *const buf, size_t size, int fl, const char *c
 	fd_set ss;
 	struct timeval tv;
 	struct sockaddr_in toAddr;
+	DECL_SIGPIPE_VARS
+	
+	if ((buf == NULL) || (size == 0) || (toAddrStr == NULL) || (toAddrStr[0] == '\0') || (tlen <= 0)) {
+		errno = EINVAL;
+		return (-1);
+	}
 
 	if ((result = AddrStrToAddr(toAddrStr, &toAddr, -1)) < 0) {
 		return (result);
@@ -124,9 +59,11 @@ SSendtoByName(int sfd, const char *const buf, size_t size, int fl, const char *c
 			time(&now);
 		}
 
+		IGNORE_SIGPIPE
 		nwrote = sendto(sfd, buf, (send_size_t) size, fl,
 			(struct sockaddr *) &toAddr,
 			(sockaddr_size_t) sizeof(struct sockaddr_in));
+		RESTORE_SIGPIPE
 
 		if (nwrote >= 0)
 			break;
@@ -137,7 +74,6 @@ SSendtoByName(int sfd, const char *const buf, size_t size, int fl, const char *c
 	return ((int) nwrote);
 }	/* SSendto */
 
-#endif
 
 
 
@@ -146,16 +82,25 @@ SendtoByName(int sfd, const char *const buf, size_t size, const char *const toAd
 {
 	int result;
 	struct sockaddr_in toAddr;
+	DECL_SIGPIPE_VARS
+	
+	if ((buf == NULL) || (size == 0) || (toAddrStr == NULL)) {
+		errno = EINVAL;
+		return (-1);
+	}
+	
 
 	if ((result = AddrStrToAddr(toAddrStr, &toAddr, -1)) < 0) {
 		return (result);
 	}
 
+	IGNORE_SIGPIPE
 	do {
 		result = (int) sendto(sfd, buf, (send_size_t) size, 0,
 				(struct sockaddr *) &toAddr,
 				(sockaddr_size_t) sizeof(struct sockaddr_in));
 	} while ((result < 0) && (errno == EINTR));
+	RESTORE_SIGPIPE
 
 	return (result);
 }	/* SendtoByName */
