@@ -1,6 +1,6 @@
 /* syshdrs.h
  * 
- * Copyright (c) 1992-1999 by Mike Gleason, NCEMRSoft.
+ * Copyright (c) 1996-2004 Mike Gleason, NcFTP Software.
  * All rights reserved.
  * 
  */
@@ -105,15 +105,28 @@
 	extern char *optarg;
 #endif
 
-#ifndef FOPEN_READ_TEXT
-#	define FOPEN_READ_TEXT "r"
-#	define FOPEN_WRITE_TEXT "w"
-#	define FOPEN_APPEND_TEXT "a"
-#endif
-#ifndef FOPEN_READ_BINARY
-#	define FOPEN_READ_BINARY "r"
-#	define FOPEN_WRITE_BINARY "w"
-#	define FOPEN_APPEND_BINARY "a"
+#ifdef __CYGWIN__
+#	ifndef FOPEN_READ_TEXT
+#		define FOPEN_READ_TEXT "rt"
+#		define FOPEN_WRITE_TEXT "wt"
+#		define FOPEN_APPEND_TEXT "at"
+#	endif
+#	ifndef FOPEN_READ_BINARY
+#		define FOPEN_READ_BINARY "rb"
+#		define FOPEN_WRITE_BINARY "wb"
+#		define FOPEN_APPEND_BINARY "ab"
+#	endif
+#else
+#	ifndef FOPEN_READ_TEXT
+#		define FOPEN_READ_TEXT "r"
+#		define FOPEN_WRITE_TEXT "w"
+#		define FOPEN_APPEND_TEXT "a"
+#	endif
+#	ifndef FOPEN_READ_BINARY
+#		define FOPEN_READ_BINARY "r"
+#		define FOPEN_WRITE_BINARY "w"
+#		define FOPEN_APPEND_BINARY "a"
+#	endif
 #endif
 
 #if defined(MACOSX) || defined(BSDOS)
@@ -285,7 +298,7 @@
 #	define Open open
 #endif
 
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	define Stat WinStat64
 #	define Lstat WinStat64
 #	define Fstat WinFStat64
